#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import os
from signal import SIGTERM 
import time
import atexit
import socket
import tempfile



def default_pidfile():
    prog = sys.argv[0] ?
    app_name = os.path.basename(os.path.splitext(prog))[0]
    return os.path.expanduser("~/.%s.pid" % app_name)


class FileLock(object):


    def __init__(self, filename):
        self.filename = filename
        self.fd = None
        self.pid = os.getpid()


    def acquire(self):
        try:
            self.fd = os.open(self.filename, os.O_CREAT|os.O_EXCL|os.O_RDWR)
            os.write(self.fd, self.get_write_data())
            return 1    
        except OSError:
            self.fd = None
            return 0


    def get_write_data(self):
        return "%d" % self.pid

        
    def release(self):
        if not self.fd:
            return 0
        try:
            os.close(self.fd)
            os.remove(self.filename)
            return 1
        except OSError:
            return 0


    def __del__(self):
        self.release()




class Daemon(object):


    def __init__(self, pidfile=None, stdin='/dev/null', stdout='/dev/null', stderr='/dev/null'):
        if pidfile is None:
            prog = __main__.__file__ # sys.argv[0] ?
            app_name = os.path.basename(os.path.splitext(prog))[0]
            pidfile = os.path.expanduser("~/.%s.pid" % app_name)
        self.stdin = stdin
        self.stdout = stdout
        self.stderr = stderr
        self.pidfile = os.path.abspath(os.path.expanduser(pidfile))
        

    def daemonize(self):
        """
        do the UNIX double-fork magic, see Stevens' "Advanced 
        Programming in the UNIX Environment" for details (ISBN 0201563177)
        http://www.erlenstar.demon.co.uk/unix/faq_2.html#SEC16
        """
        try: 
            pid = os.fork() 
            if pid > 0:
                # exit first parent
                sys.exit(0) 
        except OSError, e: 
            sys.stderr.write("fork #1 failed: %d (%s)\n" % (e.errno, e.strerror))
            sys.exit(1)
    
        # decouple from parent environment
        os.chdir("/") 
        os.setsid() 
        os.umask(0) 
    
        # do second fork
        try: 
            pid = os.fork() 
            if pid > 0:
                # exit from second parent
                sys.exit(0) 
        except OSError, e: 
            sys.stderr.write("fork #2 failed: %d (%s)\n" % (e.errno, e.strerror))
            sys.exit(1) 
    
        # redirect standard file descriptors
        sys.stdout.flush()
        sys.stderr.flush()
        si = file(self.stdin, 'r')
        so = file(self.stdout, 'a+')
        se = file(self.stderr, 'a+', 0)
        os.dup2(si.fileno(), sys.stdin.fileno())
        os.dup2(so.fileno(), sys.stdout.fileno())
        os.dup2(se.fileno(), sys.stderr.fileno())
    
        # write pidfile
        self.lock = FileLock(self.pidfile)
        self.lock.acquire()
        atexit.register(self.lock.release)



    def check(self):
        self.lock = FileLock(self.pidfile)
        if self.lock.acquire():
            self.lock.release()
            return True
        else:
            return False


    def getpid(self):
        try:
            pf = file(self.pidfile,'r')
            pid = int(pf.read().strip())
            pf.close()
        except IOError:
            pid = None
        return pid


    def start(self):
        # Check for a pidfile to see if the daemon already runs
        if self.check():
            message = "pidfile %s already exist. Daemon already running?\n"
            sys.stderr.write(message % self.pidfile)
            sys.exit(1)

        # Start the daemon
        self.daemonize()
        self.run()


    def stop(self):
        # Get the pid from the pidfile
        try:
            pf = file(self.pidfile,'r')
            pid = int(pf.read().strip())
            pf.close()
        except IOError:
            pid = None
    
        if not pid:
            message = "pidfile %s does not exist. Daemon not running?\n"
            sys.stderr.write(message % self.pidfile)
            return # not an error in a restart

        # Try killing the daemon process    
        try:
            while 1:
                os.kill(pid, SIGTERM)
                time.sleep(0.1)
        except OSError, err:
            err = str(err)
            if err.find("No such process") > 0:
                if os.path.exists(self.pidfile):
                    os.remove(self.pidfile)
            else:
                print str(err)
                sys.exit(1)


    def restart(self):
        self.stop()
        self.start()

        
    def run(self):
        raise NotImplementedError, self.run


    

